﻿using System;
using System.Collections.Generic;

namespace TestProject
{
    /// <summary>
    /// 그래프 헬퍼
    /// </summary>
    public static class GraphHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 최단 경로 함수 구하기 - GetShortestPathFunction<T>(graph, startItem)

        /// <summary>
        /// 최단 경로 함수 구하기
        /// </summary>
        /// <typeparam name="T">항목 타입</typeparam>
        /// <param name="graph">그래프</param>
        /// <param name="startItem">시작 항목</param>
        /// <returns>최단 경로 함수</returns>
        public static Func<T, IEnumerable<T>> GetShortestPathFunction<T>(Graph<T> graph, T startItem)
        {
            Dictionary<T, T> previousDictionary = new Dictionary<T, T>();
        
            Queue<T> queue = new Queue<T>();

            queue.Enqueue(startItem);

            while(queue.Count > 0)
            {
                T vertex = queue.Dequeue();

                foreach(T neighborVertex in graph.AdjacencyDictionary[vertex])
                {
                    if(previousDictionary.ContainsKey(neighborVertex))
                    {
                        continue;
                    }
            
                    previousDictionary[neighborVertex] = vertex;

                    queue.Enqueue(neighborVertex);
                }
            }

            Func<T, IEnumerable<T>> shortestPathFunc = vertex => {

                List<T> path = new List<T>{};

                T currentVertex = vertex;

                while(!currentVertex.Equals(startItem))
                {
                    path.Add(currentVertex);

                    currentVertex = previousDictionary[currentVertex];
                };

                path.Add(startItem);

                path.Reverse();

                return path;
            };

            return shortestPathFunc;
        }

        #endregion
    }
}